;; This is the Crude Oil Supply Domain
;;
;; This model was generated by itSIMPLE tool.
;;
;; Authors: Tiago Stegun Vaquero and Fernando Sette
;; Contact: tiago.vaquero@poli.usp.br
;; http://dlab.poli.usp.br


(define (domain Oil_Supply_Domain)
    (:requirements :typing :fluents :quantified-preconditions :durative-actions)
    (:types
      Oil_Tanker - object
      Port - object
      Tank - object
      Pipeline - object
      Pier - object
      Class_of_Crude_Oil - object
      Type_of_Crude_Oil - object
    )
    (:predicates
      (is_at ?tan - Tank ?por - Port)
      (belongs_to ?pie - Pier ?por - Port)
      (has_one_end_at ?pip - Pipeline ?por - Port)
      (composed_by ?cla - Class_of_Crude_Oil ?typ - Type_of_Crude_Oil)
      (docks_at ?oil - Oil_Tanker ?pie - Pier)
      (is_connected_to ?tan - Tank ?pip - Pipeline)
      (is_linked_to ?oil - Oil_Tanker ?tan - Tank)
      (docked ?oil - Oil_Tanker)
      (tankerConnected ?oil - Oil_Tanker)
      (tankerUnloading ?oil - Oil_Tanker)
      (tankConnected ?tan - Tank)
      (tankChangingVolume ?tan - Tank)
      (tankOperationType ?tan - Tank)
      (pipelineFree ?pip - Pipeline)
      (pipelineTransporting ?pip - Pipeline)
      (pierFree ?pie - Pier)
      (atPort ?oil - Oil_Tanker ?por - Port)
      (cl ?tan - Tank ?cla - Class_of_Crude_Oil)
      (lastOil ?pip - Pipeline ?cla - Class_of_Crude_Oil)
    )
    (:functions
      (tankerCapacity ?oil - Oil_Tanker)
      (inventory ?oil - Oil_Tanker ?oc - Type_of_Crude_Oil)
      (dockingTime ?oil - Oil_Tanker)
      (undockingTime ?oil - Oil_Tanker)
      (maxVol ?tan - Tank)
      (minVol ?tan - Tank)
      (currentVol ?tan - Tank)
      (decantingTime ?tan - Tank)
      (maxCapacity ?pie - Pier)
      (interfaceCosts ?cla - Class_of_Crude_Oil ?cl - Class_of_Crude_Oil)
      (valueAtPort ?cla - Class_of_Crude_Oil)
      (valueAtRefinery ?cla - Class_of_Crude_Oil ?ol - Pipeline)
      (classFlowRate ?cla - Class_of_Crude_Oil)
      (oilFlowRate ?typ - Type_of_Crude_Oil)
      (totalInterfaceCost)
      (totalLossAtPort)
      (totalLossAtRefinery)
      (totalNumberOfTimeActions)
      (domainMetric)
      (wIC)
      (wLP)
      (wLR)
      (wNA)
      (maxClassValue)
      (volumeNeeded)
      (volumeSent)
    )
    (:durative-action dock
     :parameters (?p - Pier ?n - Oil_Tanker ?pt - Port)
     :duration (= ?duration (dockingTime ?n))
     :condition 
       (at start 
         (and
           (not (docked ?n))
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (atPort ?n ?pt)
           (pierFree ?p)
           (>= (maxCapacity ?p) (tankerCapacity ?n))
           (belongs_to ?p ?pt)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (docks_at ?n ?p)
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
           (not (pierFree ?p))
         )
       )

    )

    (:durative-action undock
     :parameters (?p - Pier ?n - Oil_Tanker ?pt - Port)
     :duration (= ?duration (undockingTime ?n))
     :condition 
       (at start 
         (and
           (docked ?n)
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (docks_at ?n ?p)
           (atPort ?n ?pt)
           (not (pierFree ?p))
         )
       )

     :effect
       (at end 
         (and
           (not (docked ?n))
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
           (pierFree ?p)
           (not (docks_at ?n ?p))
           (not (atPort ?n ?pt))
         )
       )

    )

    (:durative-action connectTankerTank
     :parameters (?n - Oil_Tanker ?t - Tank ?pt - Port)
     :duration (= ?duration 0)
     :condition 
       (at start 
         (and
           (docked ?n)
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (atPort ?n ?pt)
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (is_at ?t ?pt)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (not (tankerUnloading ?n))
           (is_linked_to ?n ?t)
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
         )
       )

    )

    (:durative-action pumpOilTankerTankFull
     :parameters (?n - Oil_Tanker ?t - Tank ?ol - Type_of_Crude_Oil ?class - Class_of_Crude_Oil)
     :duration (= ?duration (/ (inventory ?n ?ol) (oilFlowRate ?ol)))
     :condition 
       (at start 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (not (tankerUnloading ?n))
           (is_linked_to ?n ?t)
           (<= (inventory ?n ?ol) (- (maxVol ?t) (currentVol ?t)))
           (> (inventory ?n ?ol) 0)
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
           (< (currentVol ?t) (maxVol ?t))
           (cl ?t ?class)
           (composed_by ?class ?ol)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (tankerUnloading ?n)
           (assign (inventory ?n ?ol) 0)
           (tankConnected ?t)
           (tankChangingVolume ?t)
           (tankOperationType ?t)
           (increase (currentVol ?t) (inventory ?n ?ol))
           (increase (totalLossAtPort) (* (inventory ?n ?ol) (- (maxClassValue) (valueAtPort ?class))))
           (increase (domainMetric) (* (wLP) (* (inventory ?n ?ol) (- (maxClassValue) (valueAtPort ?class)))))
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
         )
       )

    )

    (:durative-action disconnectTankerTank
     :parameters (?n - Oil_Tanker ?t - Tank)
     :duration (= ?duration 0)
     :condition 
       (at start 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (tankerUnloading ?n)
           (is_linked_to ?n ?t)
           (tankConnected ?t)
           (tankChangingVolume ?t)
           (tankOperationType ?t)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
           (not (is_linked_to ?n ?t))
         )
       )

    )

    (:durative-action connectTankPipeline
     :parameters (?t - Tank ?o - Pipeline ?pt - Port)
     :duration (= ?duration 0)
     :condition 
       (at start 
         (and
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (is_at ?t ?pt)
           (pipelineFree ?o)
           (not (pipelineTransporting ?o))
           (has_one_end_at ?o ?pt)
         )
       )

     :effect
       (at end 
         (and
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (is_connected_to ?t ?o)
           (not (pipelineFree ?o))
           (not (pipelineTransporting ?o))
         )
       )

    )

    (:durative-action pumpTankPipeline
     :parameters (?t - Tank ?o - Pipeline ?class - Class_of_Crude_Oil ?pipelineClass - Class_of_Crude_Oil)
     :duration (= ?duration (/ (- (currentVol ?t) (minVol ?t)) (classFlowRate ?class)))
     :condition 
       (at start 
         (and
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (is_connected_to ?t ?o)
           (cl ?t ?class)
           (not (pipelineFree ?o))
           (not (pipelineTransporting ?o))
           (lastOil ?o ?pipelineClass)
         )
       )

     :effect
       (at end 
         (and
           (tankConnected ?t)
           (tankChangingVolume ?t)
           (not (tankOperationType ?t))
           (assign (currentVol ?t) (minVol ?t))
           (increase (totalLossAtRefinery) (* (- (currentVol ?t) (minVol ?t)) (- (maxClassValue) (valueAtRefinery ?class ?o))))
           (increase (volumeSent) (- (currentVol ?t) (minVol ?t)))
           (increase (domainMetric) (* (* (wLR) (- (currentVol ?t) (minVol ?t))) (- (maxClassValue) (valueAtRefinery ?class ?o))))
           (not (pipelineFree ?o))
           (pipelineTransporting ?o)
           (increase (totalInterfaceCost) (interfaceCosts ?pipelineClass ?class))
           (increase (domainMetric) (* (wIC) (interfaceCosts ?pipelineClass ?class)))
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
           (not (lastOil ?o ?pipelineClass))
         )
       )

    )

    (:durative-action disconnectTankPipeline
     :parameters (?t - Tank ?o - Pipeline ?class - Class_of_Crude_Oil)
     :duration (= ?duration 0)
     :condition 
       (at start 
         (and
           (tankConnected ?t)
           (tankChangingVolume ?t)
           (not (tankOperationType ?t))
           (is_connected_to ?t ?o)
           (cl ?t ?class)
           (not (pipelineFree ?o))
           (pipelineTransporting ?o)
         )
       )

     :effect
       (at end 
         (and
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (pipelineFree ?o)
           (not (pipelineTransporting ?o))
           (lastOil ?o ?class)
           (not (is_connected_to ?t ?o))
         )
       )

    )

    (:durative-action connectTankerDecantingTank
     :parameters (?n - Oil_Tanker ?t - Tank ?pt - Port)
     :duration (= ?duration 0)
     :condition 
       (at start 
         (and
           (docked ?n)
           (not (tankerConnected ?n))
           (not (tankerUnloading ?n))
           (atPort ?n ?pt)
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
           (is_at ?t ?pt)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (not (tankerUnloading ?n))
           (is_linked_to ?n ?t)
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
         )
       )

    )

    (:durative-action pumpOilTankerTankParcial
     :parameters (?n - Oil_Tanker ?t - Tank ?ol - Type_of_Crude_Oil ?class - Class_of_Crude_Oil)
     :duration (= ?duration (/ (- (maxVol ?t) (currentVol ?t)) (oilFlowRate ?ol)))
     :condition 
       (at start 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (not (tankerUnloading ?n))
           (is_linked_to ?n ?t)
           (> (inventory ?n ?ol) (- (maxVol ?t) (currentVol ?t)))
           (tankConnected ?t)
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
           (< (currentVol ?t) (maxVol ?t))
           (cl ?t ?class)
           (composed_by ?class ?ol)
         )
       )

     :effect
       (at end 
         (and
           (docked ?n)
           (tankerConnected ?n)
           (tankerUnloading ?n)
           (decrease (inventory ?n ?ol) (- (maxVol ?t) (currentVol ?t)))
           (tankConnected ?t)
           (tankChangingVolume ?t)
           (tankOperationType ?t)
           (assign (currentVol ?t) (maxVol ?t))
           (increase (totalLossAtPort) (* (- (maxVol ?t) (currentVol ?t)) (- (maxClassValue) (valueAtPort ?class))))
           (increase (domainMetric) (* (* (wLP) (- (maxVol ?t) (currentVol ?t))) (- (maxClassValue) (valueAtPort ?class))))
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
         )
       )

    )

    (:durative-action waitForDecantation
     :parameters (?t - Tank)
     :duration (= ?duration (decantingTime ?t))
     :condition 
       (at start 
         (and
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (tankOperationType ?t)
         )
       )

     :effect
       (at end 
         (and
           (not (tankConnected ?t))
           (not (tankChangingVolume ?t))
           (not (tankOperationType ?t))
           (increase (totalNumberOfTimeActions) 1)
           (increase (domainMetric) (* (wNA) 1))
         )
       )

    )

)
